<img align="right" src="assets/hook-and-loop.gif">

# React Loops — _React Veclro Architecture_

React Loops work alongside React Hooks as part of the novel _React Velcro_
architecture for building sticky, secure user interfaces that don't come apart
under pressure.

## For Loops

Use the props `of` to provide the list and `as` to provide an element for
each item in the list. The `of` prop accepts Arrays, Array-likes,
and Iterables.

```js
<ul>
  <For of={myList} as={item =>
    <li>{item}</li>
  }/>
</ul>
```

Or provide a "render prop" function as a child.

```js
<ul>
  <For of={myList}>
    {item =>
      <li>{item}</li>
    }
  </For>
</ul>
```

Access additional information about each iteration by destructuring the
second argument:

- `index`: A number from 0 to the length of the list
- `length`: The length of the list
- `key`: The key for this item in the list. Same as `index` for Arrays
         but string properties for `in` Objects
- `isFirst`: True for the first iteration
- `isLast`: True for the last iteration

```js
<ul>
  <For of={myList} as={(item, { isLast }) =>
    <li><If case={isLast}>and </If>{item}</li>
  }/>
</ul>
```

### For in Loops

Use the prop `in` to provide an Object instead of an Array or Iterable.

```js
<ul>
  <For in={myObj} as={(item, {key}) =>
    <li>{key}: {item}</li>
  }/>
</ul>
```

### React Keys

Provide `key` on each child to ensure correct behavior if the list may be
reordered over time. If you don't provide `key`, the `key` of each
iteration will be used by default.

```js
<ul>
  <For of={myList} as={item =>
    <li key={item.id}>{item.label}</li>
  }/>
</ul>
```


## If conditions

Use the `case` prop with `<If>` and `<ElseIf>` elements to conditionally
include certain elements. When an `<If>` case is _truthy_ it does not
render any `<ElseIf>` or `<Else>` children. However when it is _falsey_ it
_only_ renders `<ElseIf>` and `<Else>` children.

```js
<If case={someCondition}>
  This will only be shown if someCondition is truthy.
  <ElseIf case={otherCondition}>
    This will only be shown if someCondition is falsey
    and otherCondition is truthy.
    <Else>
      This will only be shown if both someCondition and
      otherCondition are both falsey.
    </Else>
  </ElseIf>
  <Else>
    This will be shown if someCondition is falsey.
    <If case={finalCondition}>
      This will be shown if someCondition is falsey
      and finalCondition is truthy.
    </If>
  </Else>
</If>
```
